@@ -65,7 +65,12 @@ class JsonPath(object):
                 result.extend(obj)
                 self.start_parsing(obj, expr, result)
             elif expr.startswith(".[?"):
-                expr = expr.replace(".[?", "..[?")
+                expr = "..[?" + expr[3:]
+                obj, expr = self.scan_parsing(obj, expr)
+                result.extend(obj)
+                self.start_parsing(obj, expr, result)
+            elif expr.startswith(".["):
+                expr = "..[" + expr[2:]
                 obj, expr = self.scan_parsing(obj, expr)
                 result.extend(obj)
                 self.start_parsing(obj, expr, result)
@@ -140,7 +145,7 @@ class JsonPath(object):
                 result.append(value)
                 for item in value.values():
                     scan(item, x)
-            elif x == '*' or x == '[*]':
+            elif x == '*' or x == '[*]' or value is None:
                 result.append(value)
 
         g = re.match(pattern_double_dot, expr)
@@ -280,10 +285,9 @@ class JsonPath(object):
             return result
 
         for item in obj:
-            if not isinstance(item, list):
-                continue
-
-            for child in item:
+            value = None
+            if isinstance(item, dict):
+                child = deepcopy(item)
                 try:
                     value = eval(expr)
                 except Exception:
@@ -292,6 +296,16 @@ class JsonPath(object):
                 if value:
                     result.append(child)
 
+            elif isinstance(item, list):
+                for child in item:
+                    try:
+                        value = eval(expr)
+                    except Exception:
+                        continue
+
+                    if value:
+                        result.append(child)
+
         return result
 
     def controller_parsing(self, obj, expr):
